#!/usr/bin/env python

"""Tests for `genotype_variants` package."""


import unittest
import pandas as pd

from genotype_variants.commands import small_variants


class TestGenotype_variants(unittest.TestCase):
    """Tests for `genotype_variants` package."""

    def setUp(self):
        """Set up test fixtures, if any."""
        self.mutation_key = ['Chromosome', 'Start_Position', 'End_Position', 'Reference_Allele', 'Tumor_Seq_Allele1']
        self.d_maf = pd.read_csv('tests/test_data/test_duplex.maf', sep="\t", header="infer")
        self.s_maf = pd.read_csv('tests/test_data/test_simplex.maf', sep="\t", header="infer")
        self.d_maf = self.d_maf.set_index(self.mutation_key, drop=False)
        self.s_maf = self.s_maf.set_index(self.mutation_key, drop=False)

    def tearDown(self):
        """Tear down test fixtures, if any."""
        pass

    def test_command_line_interface(self):
        """Test the CLI."""
        pass
        # runner = CliRunner()
        # result = runner.invoke(cli.main)
        # assert result.exit_code == 0
        # assert 'genotype_variants.cli.main' in result.output
        # help_result = runner.invoke(cli.main, ['--help'])
        # assert help_result.exit_code == 0
        # assert '--help  Show this message and exit.' in help_result.output

    def test_merge_simplex_duplex(self):
        """
        Test simplex-duplex MAF generation

        :return:
        """
        df_merge = small_variants.create_duplex_simplex_maf(self.s_maf, self.d_maf)
        df_merge = df_merge.sort_index()
        expected = pd.read_csv('tests/test_data/expected.maf', sep='\t')
        expected = expected.set_index(self.mutation_key, drop=False)
        expected = expected.sort_index()

        pd.testing.assert_frame_equal(df_merge, expected)

        # SNP
        snp_index = (1, 8080157, 8080157, 'T',   'A')
        assert df_merge.loc[snp_index]['t_ref_count_fragment'] == 1549
        assert df_merge.loc[snp_index]['t_alt_count_fragment'] == 1
        assert df_merge.loc[snp_index]['t_total_count_fragment'] == 1550
        # INS
        insertion_index = (18, 48584855, 48584855, 'A',   'TTT')
        assert df_merge.loc[insertion_index]['t_ref_count_fragment'] == 694
        assert df_merge.loc[insertion_index]['t_alt_count_fragment'] == 4
        assert df_merge.loc[insertion_index]['t_total_count_fragment'] == 698
        # DEL
        deletion_index = (18, 57571783, 57571783, 'T',   '-')
        assert df_merge.loc[deletion_index]['t_ref_count_fragment'] == 514
        assert df_merge.loc[deletion_index]['t_alt_count_fragment'] == 6
        assert df_merge.loc[deletion_index]['t_total_count_fragment'] == 520



import logging
import subprocess
"""
run_cmd
~~~~~~~~~~~~~~~
:Description: Code to run shell commands 
"""
"""
Created on January 29, 2020
Description: Code to run shell commands
@author: Ronak H Shah
"""
# Making logging possible
logger = logging.getLogger("genotype_variants")

def run_cmd(cmd):
    """Code to run shell commands"""
    logger.debug(
        "run_cmd: run: the command line is %s",
        cmd.encode("unicode_escape").decode("utf-8"),
    )
    out = subprocess.Popen(
        (cmd),
        stdin=subprocess.PIPE,
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        shell=True,
    )
    out.wait()
    stdout, stderr = out.communicate()
    if stderr is None:
        logger.debug("run_cmd: run: Read: %s", stdout.decode("utf-8"))
    else:
        logger.error(
            "run_cmd: run: could not run"
        )
    return(out)



import os
import sys
import logging
import time
import pathlib
import subprocess
import numpy as np
import re
from genotype_variants.run_cmd import run_cmd
from genotype_variants.create_duplex_simplex_dataframe import (
    create_duplex_simplex_dataframe as cdsd,
)
from genotype_variants.create_all_maf_dataframe import create_all_maf_dataframe as camd

try:
    import click
except ImportError as e:
    print(
        "small_variants: click is not installed, please install click as it is one of the requirements."
    )
    exit(1)
try:
    import click_log
except ImportError as e:
    print(
        "small_variants: click-log is not installed, please install click_log as it is one of the requirements."
    )
    exit(1)
try:
    import pandas as pd
except ImportError as e:
    print(
        "small_variants: pandas is not installed, please install pandas as it is one of the requirements."
    )
    exit(1)
"""
small_variants
~~~~~~~~~~~~~~~
:Description: console script for genotyping small variants
"""
"""
Created on January 29, 2020
Description: console script for genotyping small variants
@author: Ronak H Shah
@coauthor: Maysun Hasan
"""
BASE_DIR = pathlib.Path("__file__").resolve().parent
# Making logging possible
logger = logging.getLogger("genotype_variants")
click_log.basic_config(logger)
click_log.ColorFormatter.colors["info"] = dict(fg="green")


@click.group()
def cli():
    """Sub-commands for genotyping small variants"""
    pass


# Generate
@cli.command()
@click.option(
    "-i",
    "--input-maf",
    required=True,
    type=click.Path(exists=True),
    help="Full path to small variants input file in MAF format",
)
@click.option(
    "-r",
    "--reference-fasta",
    required=True,
    type=click.Path(exists=True),
    help="Full path to reference file in FASTA format",
)
@click.option(
    "-p",
    "--patient-id",
    required=True,
    type=click.STRING,
    help="Alphanumeric string indicating patient identifier",
)
@click.option(
    "-b",
    "--standard-bam",
    required=False,
    type=click.Path(exists=True),
    help="Full path to standard bam file, Note: This option assumes that the .bai file is present at same location as the bam file",
)
@click.option(
    "-d",
    "--duplex-bam",
    required=False,
    type=click.Path(exists=True),
    help="Full path to duplex bam file, Note: This option assumes that the .bai file is present at same location as the bam file",
)
@click.option(
    "-s",
    "--simplex-bam",
    required=False,
    type=click.Path(exists=True),
    help="Full path to simplex bam file, Note: This option assumes that the .bai file is present at same location as the bam file",
)
@click.option(
    "-g",
    "--gbcms-path",
    required=True,
    type=click.Path(exists=True),
    help="Full path to GetBaseCountMultiSample executable with fragment support",
)
@click.option(
    "-fd",
    "--filter-duplicate",
    required=False,
    default=0,
    type=click.INT,
    help="Filter duplicate parameter for GetBaseCountMultiSample",
)
@click.option(
    "-fc",
    "--fragment-count",
    required=False,
    default=1,
    type=click.INT,
    help="Fragment Count parameter for GetBaseCountMultiSample",
)
@click.option(
    "-mapq",
    "--mapping-quality",
    required=False,
    default=20,
    type=click.INT,
    help="Mapping quality for GetBaseCountMultiSample",
)
@click.option(
    "-t",
    "--threads",
    required=False,
    default=1,
    type=click.INT,
    help="Number of threads to use for GetBaseCountMultiSample",
)
@click_log.simple_verbosity_option(logger)
def generate(
    input_maf,
    reference_fasta,
    gbcms_path,
    patient_id,
    standard_bam,
    duplex_bam,
    simplex_bam,
    filter_duplicate,
    fragment_count,
    mapping_quality,
    threads,
):
    """Command that helps to generate genotyped MAF,
    the output file will be labelled with
    patient identifier as prefix"""
    pid = os.getpid()
    logger_file = "genotype_variants_" + str(pid) + ".log"
    logger_output = pathlib.Path.cwd().joinpath(logger_file)
    fh = logging.FileHandler(logger_output)
    formatter = logging.Formatter(
        fmt="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
        datefmt="%m/%d/%Y %I:%M:%S %p",
    )
    fh.setFormatter(formatter)
    logger.addHandler(fh)
    logger.info(
        "=========================================================================="
    )
    logger.info(
        ">>> Running genotype_variants for small variants to generate genotypes <<<"
    )
    logger.info(
        "=========================================================================="
    )
    t1_start = time.perf_counter()
    t2_start = time.process_time()
    if standard_bam or duplex_bam or simplex_bam:
        pass
    else:
        logger.error(
            "Required to specify at-least one input BAM file option. Please refer to the README for more information"
        )
        exit(1)

    logger.info("small_variants: Patient ID: %s", patient_id)
    logger.info("small_variants: Input MAF: %s", input_maf)
    logger.info("small_variants: Reference FASTA: %s", reference_fasta)
    if standard_bam:
        logger.info("small_variants: Standard BAM: %s", standard_bam)
    if duplex_bam:
        logger.info("small_variants: Duplex BAM: %s", duplex_bam)
    if simplex_bam:
        logger.info("small_variants: Simplex BAM: %s", simplex_bam)
    logger.info("small_variants: GetBaseCountMultiSample -> Path: %s", gbcms_path)
    logger.info(
        "small_variants: GetBaseCountMultiSample -> Filter Duplicate: %s",
        str(filter_duplicate),
    )
    logger.info(
        "small_variants: GetBaseCountMultiSample -> Fragment Count: %s",
        str(fragment_count),
    )
    logger.info(
        "small_variants: GetBaseCountMultiSample -> Mapping Quality: %s",
        str(mapping_quality),
    )
    logger.info("small_variants: GetBaseCountMultiSample -> Threads: %s", str(threads))

    # Run GetBaseMultisampleCount for each available bam file
    std_output_maf = None
    duplex_output_maf = None
    simplex_output_maf = None
    p1, p2, p3 = None, None, None
    if standard_bam:
        btype = "STANDARD"
        (cmd, std_output_maf) = generate_gbcms_cmd(
            input_maf,
            btype,
            reference_fasta,
            gbcms_path,
            patient_id,
            standard_bam,
            filter_duplicate,
            fragment_count,
            mapping_quality,
            threads,
        )
        p1 = run_cmd(cmd)
        logger.info(
            "small_variants: Done running gbcms on %s and data has been written to %s",
            standard_bam,
            std_output_maf,
        )

    if duplex_bam:
        btype = "DUPLEX"
        (cmd, duplex_output_maf) = generate_gbcms_cmd(
            input_maf,
            btype,
            reference_fasta,
            gbcms_path,
            patient_id,
            duplex_bam,
            filter_duplicate,
            fragment_count,
            mapping_quality,
            threads,
        )
        p2 = run_cmd(cmd)
        logger.info(
            "small_variants: Done running gbcms on %s and data has been written to %s",
            duplex_bam,
            duplex_output_maf,
        )

    if simplex_bam:
        btype = "SIMPLEX"
        (cmd, simplex_output_maf) = generate_gbcms_cmd(
            input_maf,
            btype,
            reference_fasta,
            gbcms_path,
            patient_id,
            simplex_bam,
            filter_duplicate,
            fragment_count,
            mapping_quality,
            threads,
        )
        p3 = run_cmd(cmd)
        logger.info(
            "small_variants: Done running gbcms on %s and data has been written to %s",
            simplex_bam,
            simplex_output_maf,
        )

    logger.info("small_variants: Completed processing based on the given instructions")

    t1_stop = time.perf_counter()
    t2_stop = time.process_time()
    logger.info("--------------------------------------------------")
    logger.info("Elapsed time: %.1f [min]" % ((t1_stop - t1_start) / 60))
    logger.info("CPU process time: %.1f [min]" % ((t2_stop - t2_start) / 60))
    logger.info("--------------------------------------------------")
    return (std_output_maf, simplex_output_maf, duplex_output_maf)


def generate_gbcms_cmd(
    input_maf,
    btype,
    reference_fasta,
    gbcms_path,
    patient_id,
    bam,
    filter_duplicate,
    fragment_count,
    mapping_quality,
    threads,
):

    """This will help generate command for GetBaseCountMultiSample"""

    sample_id = patient_id + "-" + btype
    output_maf = pathlib.Path.cwd().joinpath(sample_id + "_genotyped.maf")

    cmd = (
        str(gbcms_path)
        + " --bam "
        + sample_id
        + ":"
        + str(bam)
        + " --filter_duplicate "
        + str(filter_duplicate)
        + " --fragment_count "
        + str(fragment_count)
        + " --maf "
        + str(input_maf)
        + " --maq "
        + str(mapping_quality)
        + " --omaf"
        + " --output "
        + str(output_maf)
        + " --fasta "
        + str(reference_fasta)
        + " --thread "
        + str(threads)
    )

    return (cmd, output_maf)


# Merge
@cli.command()
@click.option(
    "-i",
    "--input-maf",
    required=False,
    type=click.Path(exists=True),
    help="Full path to small variants input file in MAF format used for input to GBCMS for generating genotypes",
)
@click.option(
    "-std",
    "--input-standard-maf",
    required=False,
    type=click.Path(exists=True),
    help="Full path to small variants input file in MAF format generated by GBCMS for standard_bam",
)
@click.option(
    "-d",
    "--input-duplex-maf",
    required=False,
    type=click.Path(exists=True),
    help="Full path to small variants input file in MAF format generated by GBCMS for duplex_bam",
)
@click.option(
    "-s",
    "--input-simplex-maf",
    required=False,
    type=click.Path(exists=True),
    help="Full path to small variants input file in MAF format generated by GBCMS for simplex_bam",
)
@click.option(
    "-p",
    "--patient-id",
    required=True,
    type=click.STRING,
    help="Alphanumeric string indicating patient identifier",
)
@click_log.simple_verbosity_option(logger)
def merge(
    patient_id, input_maf, input_standard_maf, input_duplex_maf, input_simplex_maf
):
    """
    Given original input MAF used as an input for GBCMS along with
    GBCMS generated output MAF for standard_bam, duplex_bam or simplex bam,
    Merge them into a single output MAF format.
    If both duplex_bam and simplex_bam based MAF are provided
    the program will generate merged genotypes as well.
    The output file will be based on the give alphanumeric patient identifier as prefix.
    """
    pid = os.getpid()
    logger_file = "genotype_variants_" + str(pid) + ".log"
    logger_output = pathlib.Path.cwd().joinpath(logger_file)
    fh = logging.FileHandler(logger_output)
    formatter = logging.Formatter(
        fmt="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
        datefmt="%m/%d/%Y %I:%M:%S %p",
    )
    fh.setFormatter(formatter)
    logger.addHandler(fh)
    logger.info(
        "========================================================================"
    )
    logger.info(
        ">>> Running genotype_variants for small variants to merge MAF output <<<"
    )
    logger.info(
        "========================================================================"
    )
    t1_start = time.perf_counter()
    t2_start = time.process_time()

    number_of_variables = 0
    for status in (input_maf, input_standard_maf, input_duplex_maf, input_simplex_maf):
        if status:
            number_of_variables += 1
    if number_of_variables >= 2:
        pass
    else:
        logger.error(
            "genotype_variants:small_variants:merge:: At least two MAF input need to be provided for us to merge."
        )
        exit(1)
    o_maf, i_maf, d_maf, s_maf = None, None, None, None
    if input_maf:
        logger.info(
            "genotype_variants:small_variants:merge:: Original MAF -> %s", input_maf
        )
        o_maf = pd.read_csv(input_maf, sep="\t", header="infer", index_col=False)
    if input_standard_maf:
        logger.info(
            "genotype_variants:small_variants:merge:: STANDARD BAM MAF -> %s",
            input_standard_maf,
        )

        create_empty_maf_if_missing(input_standard_maf)

        i_maf = pd.read_csv(
            input_standard_maf, sep="\t", header="infer", index_col=False
        )
    if input_duplex_maf:

        create_empty_maf_if_missing(input_duplex_maf)

        d_maf = pd.read_csv(input_duplex_maf, sep="\t", header="infer", index_col=False)
        logger.info(
            "genotype_variants:small_variants:merge:: DUPLEX BAM MAF -> %s",
            input_duplex_maf,
        )
    if input_simplex_maf:

        create_empty_maf_if_missing(input_simplex_maf)

        s_maf = pd.read_csv(
            input_simplex_maf, sep="\t", header="infer", index_col=False
        )
        logger.info(
            "genotype_variants:small_variants:merge:: SIMPLEX BAM MAF -> %s",
            input_simplex_maf,
        )

    # generate duplex simplex data frame
    ds_maf = None

    if d_maf is not None and s_maf is not None:
        ds_maf = cdsd(s_maf, d_maf)
        file_name = pathlib.Path.cwd().joinpath(
            patient_id + "-SIMPLEX-DUPLEX" + "_genotyped.maf"
        )
        write_csv(file_name, ds_maf)

    # generate data frame based on satisfying conditions
    file_name = None
    (df_o_s_ds, df_o_s, df_s_ds, df_s_ds) = None, None, None, None
    if o_maf is not None and i_maf is not None and ds_maf is not None:
        df_o_s_ds = camd(o_maf, i_maf, ds_maf)
        file_name = pathlib.Path.cwd().joinpath(
            patient_id + "-ORG-STD-SIMPLEX-DUPLEX" + "_genotyped.maf"
        )
        write_csv(file_name, df_o_s_ds)
    elif o_maf is not None and i_maf is not None:
        df_o_s = camd(o_maf, i_maf, None)
        file_name = pathlib.Path.cwd().joinpath(
            patient_id + "-ORG-STD" + "_genotyped.maf"
        )
        write_csv(file_name, df_o_s)
    elif o_maf is not None and ds_maf is not None:
        df_o_ds = camd(o_maf, None, ds_maf)
        file_name = pathlib.Path.cwd().joinpath(
            patient_id + "-ORG-SIMPLEX-DUPLEX" + "_genotyped.maf"
        )
        write_csv(file_name, df_o_ds)
    elif i_maf is not None and ds_maf is not None:
        df_s_ds = camd(None, i_maf, ds_maf)
        file_name = pathlib.Path.cwd().joinpath(
            patient_id + "-STD-SIMPLEX-DUPLEX" + "_genotyped.maf"
        )
        write_csv(file_name, df_s_ds)
    elif i_maf is not None and d_maf is not None:
        pass
    elif i_maf is not None and s_maf is not None:
        pass
    elif o_maf is not None and d_maf is not None:
        pass
    elif o_maf is not None and s_maf is not None:
        pass
    else:
        file_name = pathlib.Path.cwd().joinpath(
            patient_id + "-SIMPLEX-DUPLEX" + "_genotyped.maf"
        )
        write_csv(file_name, ds_maf)
    t1_stop = time.perf_counter()
    t2_stop = time.process_time()
    logger.info("--------------------------------------------------")
    logger.info("Elapsed time: %.1f [min]" % ((t1_stop - t1_start) / 60))
    logger.info("CPU process time: %.1f [min]" % ((t2_stop - t2_start) / 60))
    logger.info("--------------------------------------------------")
    return file_name


def create_empty_maf_if_missing(filename):
    header = [
        'Hugo_Symbol',
        'Entrez_Gene_Id',
        'Center',
        'NCBI_Build',
        'Chromosome',
        'Start_Position',
        'End_Position',
        'Strand',
        'Variant_Classification',
        'Variant_Type',
        'Reference_Allele',
        'Tumor_Seq_Allele1',
        'Tumor_Seq_Allele2',
        'dbSNP_RS',
        'dbSNP_Val_Status',
        'Tumor_Sample_Barcode',
        'Matched_Norm_Sample_Barcode',
        'Match_Norm_Seq_Allele1',
        'Match_Norm_Seq_Allele2',
        'Tumor_Validation_Allele1',
        'Tumor_Validation_Allele2',
        'Match_Norm_Validation_Allele1',
        'Match_Norm_Validation_Allele2',
        'Verification_Status',
        'Validation_Status',
        'Mutation_Status',
        'Sequencing_Phase',
        'Sequence_Source',
        'Validation_Method',
        'Score',
        'BAM_File',
        'Sequencer',
        't_ref_count',
        't_alt_count',
        'n_ref_count',
        'n_alt_count',
        'Caller',
        't_total_count',
        't_variant_frequency',
        't_total_count_forward',
        't_ref_count_forward',
        't_alt_count_forward',
        't_total_count_fragment',
        't_ref_count_fragment',
        't_alt_count_fragment']

    if not os.path.exists(filename):
        empty_df = pd.DataFrame(columns=header)
        empty_df.to_csv(filename, index=False, sep='\t')


def write_csv(file_name, data_frame):
    try:
        data_frame.to_csv(str(file_name), sep="\t", index=False)
        logger.info(
            "genotype_variants:small_variants:create_csv:: merged genotyped data has been written to %s",
            file_name,
        )
    except:
        e = sys.exc_info()[0]
        logger.error(
            "genotype_variants:small_variants:create_csv:: could not write to CSV file, due to error: %s",
            e,
        )
        exit(1)


# All
@cli.command()
@click.option(
    "-i",
    "--input-maf",
    required=True,
    type=click.Path(exists=True),
    help="Full path to small variants input file in MAF format",
)
@click.option(
    "-r",
    "--reference-fasta",
    required=True,
    type=click.Path(exists=True),
    help="Full path to reference file in FASTA format",
)
@click.option(
    "-p",
    "--patient-id",
    required=True,
    type=click.STRING,
    help="Alphanumeric string indicating patient identifier",
)
@click.option(
    "-b",
    "--standard-bam",
    required=False,
    type=click.Path(exists=True),
    help="Full path to standard bam file, Note: This option assumes that the .bai file is present at same location as the bam file",
)
@click.option(
    "-d",
    "--duplex-bam",
    required=False,
    type=click.Path(exists=True),
    help="Full path to duplex bam file, Note: This option assumes that the .bai file is present at same location as the bam file",
)
@click.option(
    "-s",
    "--simplex-bam",
    required=False,
    type=click.Path(exists=True),
    help="Full path to simplex bam file, Note: This option assumes that the .bai file is present at same location as the bam file",
)
@click.option(
    "-g",
    "--gbcms-path",
    required=True,
    type=click.Path(exists=True),
    help="Full path to GetBaseCountMultiSample executable with fragment support",
)
@click.option(
    "-fd",
    "--filter-duplicate",
    required=False,
    default=0,
    type=click.INT,
    help="Filter duplicate parameter for GetBaseCountMultiSample",
)
@click.option(
    "-fc",
    "--fragment-count",
    required=False,
    default=1,
    type=click.INT,
    help="Fragment Count parameter for GetBaseCountMultiSample",
)
@click.option(
    "-mapq",
    "--mapping-quality",
    required=False,
    default=20,
    type=click.INT,
    help="Mapping quality for GetBaseCountMultiSample",
)
@click.option(
    "-t",
    "--threads",
    required=False,
    default=1,
    type=click.INT,
    help="Number of threads to use for GetBaseCountMultiSample",
)
@click_log.simple_verbosity_option(logger)
def all(
    input_maf,
    reference_fasta,
    gbcms_path,
    patient_id,
    standard_bam,
    duplex_bam,
    simplex_bam,
    filter_duplicate,
    fragment_count,
    mapping_quality,
    threads,
):
    """
    Command that helps to generate genotyped MAF and
    merge the genotyped MAF.
    the output file will be labelled with
    patient identifier as prefix
    """
    pid = os.getpid()
    logger_file = "genotype_variants_" + str(pid) + ".log"
    logger_output = pathlib.Path.cwd().joinpath(logger_file)
    fh = logging.FileHandler(logger_output)
    formatter = logging.Formatter(
        fmt="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
        datefmt="%m/%d/%Y %I:%M:%S %p",
    )
    fh.setFormatter(formatter)
    logger.addHandler(fh)
    logger.info(
        "========================================================================================"
    )
    logger.info(
        ">>> Running genotype_variants for small variants to generate genotypes and merge MAF <<<"
    )
    logger.info(
        "========================================================================================="
    )
    t1_start = time.perf_counter()
    t2_start = time.process_time()
    (standard_maf, simplex_maf, duplex_maf) = generate.callback(
        input_maf,
        reference_fasta,
        gbcms_path,
        patient_id,
        standard_bam,
        duplex_bam,
        simplex_bam,
        filter_duplicate,
        fragment_count,
        mapping_quality,
        threads,
    )
    final_file = merge.callback(
        patient_id, input_maf, standard_maf, duplex_maf, simplex_maf
    )
    t1_stop = time.perf_counter()
    t2_stop = time.process_time()
    logger.info("--------------------------------------------------")
    logger.info("Elapsed time: %.1f [min]" % ((t1_stop - t1_start) / 60))
    logger.info("CPU process time: %.1f [min]" % ((t2_stop - t2_start) / 60))
    logger.info("--------------------------------------------------")
    return final_file


# Multiple Sample Process
@cli.command()
@click.option(
    "-i",
    "--input-metadata",
    required=True,
    type=click.Path(exists=True),
    help="Full path to metadata file in TSV/EXCEL format, with following headers: sample_id, maf, standard_bam, duplex_bam, simplex_bam. Make sure to use full paths inside the metadata file",
)
@click.option(
    "-r",
    "--reference-fasta",
    required=True,
    type=click.Path(exists=True),
    help="Full path to reference file in FASTA format",
)
@click.option(
    "-g",
    "--gbcms-path",
    required=True,
    type=click.Path(exists=True),
    help="Full path to GetBaseCountMultiSample executable with fragment support",
)
@click.option(
    "-fd",
    "--filter-duplicate",
    required=False,
    default=0,
    type=click.INT,
    help="Filter duplicate parameter for GetBaseCountMultiSample",
)
@click.option(
    "-fc",
    "--fragment-count",
    required=False,
    default=1,
    type=click.INT,
    help="Fragment Count parameter for GetBaseCountMultiSample",
)
@click.option(
    "-mapq",
    "--mapping-quality",
    required=False,
    default=20,
    type=click.INT,
    help="Mapping quality for GetBaseCountMultiSample",
)
@click.option(
    "-t",
    "--threads",
    required=False,
    default=1,
    type=click.INT,
    help="Number of threads to use for GetBaseCountMultiSample",
)
@click_log.simple_verbosity_option(logger)
def multiple_samples(
    input_metadata,
    reference_fasta,
    gbcms_path,
    filter_duplicate,
    fragment_count,
    mapping_quality,
    threads,
):
    """
    Command that helps to generate genotyped MAF and
    merge the genotyped MAF for multiple samples.
    the output file will be labelled with
    patient identifier as prefix

    Expected header of metadata_file in any order:
    sample_id,
    maf,
    standard_bam,
    duplex_bam,
    simplex_bam

    For maf, standard_bam, duplex_bam and simplex_bam please include full path to the file.
    """
    pid = os.getpid()
    logger_file = "genotype_variants_" + str(pid) + ".log"
    logger_output = pathlib.Path.cwd().joinpath(logger_file)
    fh = logging.FileHandler(logger_output)
    formatter = logging.Formatter(
        fmt="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
        datefmt="%m/%d/%Y %I:%M:%S %p",
    )
    fh.setFormatter(formatter)
    logger.addHandler(fh)
    logger.info(
        "========================================================================================"
    )
    logger.info(
        ">>> Running genotype_variants for small variants to generate genotypes and merge MAF <<<"
    )
    logger.info(
        "========================================================================================="
    )
    t1_start = time.perf_counter()
    t2_start = time.process_time()
    metadata = None
    try:
        metadata = pd.read_excel(input_metadata)
    except:
        e = sys.exc_info()[0]
        logger.warning(
            "genotype_variants:small_variants:multiple_samples:: could not read to EXCEL file, due to error: %s",
            e,
        )
        logger.warning(
            "genotype_variants:small_variants:multiple_samples:: Assuming its as TSV file"
        )
        pass
    if metadata is None:
        try:
            metadata = pd.read_csv(input_metadata, sep="\t", header="infer")
        except:
            e = sys.exc_info()[0]
            logger.error(
                "genotype_variants:small_variants:multiple_samples:: could not read TSV file, due to error: %s. Please fix and rerun the script",
                e,
            )
            exit(1)
    else:
        pass
    for ind in metadata.index:
        if pd.notnull(metadata["maf"][ind]):
            if pathlib.Path(metadata["maf"][ind]).is_file():
                input_maf = metadata["maf"][ind]
            else:
                logger.error(
                    "genotype_variants::small_variants::multiple_samples:: Maf file to genotype variants is present but the path is invalid. Please provide a valid path"
                )
                exit(1)
        else:
            logger.error(
                "genotype_variants::small_variants::multiple_samples:: Maf file to genotype variants is not present and is required."
            )
            exit(1)
        if pd.notnull(metadata["standard_bam"][ind]):
            if pathlib.Path(metadata["standard_bam"][ind]).is_file():
                standard_bam = metadata["standard_bam"][ind]
            else:
                standard_bam = None
        else:
            standard_bam = None
            logger.info(
                "genotype_variants::small_variants::multiple_samples:: Standard BAM file to genotype variants is not present."
            )
        if pd.notnull(metadata["duplex_bam"][ind]):
            if pathlib.Path(metadata["duplex_bam"][ind]).is_file():
                duplex_bam = metadata["duplex_bam"][ind]
            else:
                duplex_bam = None
        else:
            duplex_bam = None
        if pd.notnull(metadata["simplex_bam"][ind]):
            if pathlib.Path(metadata["simplex_bam"][ind]).is_file():
                simplex_bam = metadata["simplex_bam"][ind]
            else:
                simplex_bam = None
        else:
            simplex_bam = None

        if standard_bam or duplex_bam or simplex_bam:
            logger.info(
                "genotype_variants::small_variants::multiple_samples:: standard_bam, duplex_bam and simplex_bam are present for genotype variants."
            )
        else:
            logger.warning(
                "genotype_variants::small_variants::multiple_samples:: one of standard_bam, duplex_bam and simplex_bam is not present for genotype variants! Either the Standard BAM or the Duplex BAM and the Simplex BAM should be present for genotype variants."
            )

        if pd.notnull(metadata["sample_id"][ind]):
            sample_id = metadata["sample_id"][ind]
        else:
            logger.error(
                "genotype_variants:small_variants:multiple_samples:: Sample id is not a string, please check input metadata file and try again."
            )
            exit(1)
        logger.info(
            "genotype_variants:small_variants::multiple_samples:: %s is being processed",
            sample_id,
        )
        final_file = all.callback(
            input_maf,
            reference_fasta,
            gbcms_path,
            sample_id,
            standard_bam,
            duplex_bam,
            simplex_bam,
            filter_duplicate,
            fragment_count,
            mapping_quality,
            threads,
        )
    t1_stop = time.perf_counter()
    t2_stop = time.process_time()
    logger.info("--------------------------------------------------")
    logger.info("Elapsed time: %.1f [min]" % ((t1_stop - t1_start) / 60))
    logger.info("CPU process time: %.1f [min]" % ((t2_stop - t2_start) / 60))
    logger.info("--------------------------------------------------")
    return



import logging
import numpy as np

"""
generate_summary_field
~~~~~~~~~~~~~~~
:Description: Code to Generate Summary Field
"""
"""
Created on February 10, 2020
Description: Code to Generate Summary Field
@author: Ronak H Shah & Maysun Hasan
"""
# Making logging possible
logger = logging.getLogger("genotype_variants")


def generate_summary_field(df_fillout):
    """Code to Generate Summary Field"""
    np.seterr(divide="ignore", invalid="ignore")
    df_fillout = df_fillout.copy()
    df_fillout["summary_fragment"] = (
        "DP="
        + (
            df_fillout["t_alt_count_fragment"].astype(int)
            + df_fillout["t_ref_count_fragment"].astype(int)
        ).astype(str)
        + ";RD="
        + df_fillout["t_ref_count_fragment"].astype(str)
        + ";AD="
        + df_fillout["t_alt_count_fragment"].astype(str)
        + ";VF="
        + df_fillout["t_vaf_fragment"].fillna(0).astype(str)
    )
    return df_fillout



# -*- coding: utf-8 -*-
import os
import sys
import pathlib
import logging
try:
    import click
except ImportError as e:
    print(
        "cli: click is not installed, please install click as it is one of the requirements."
    )
    exit(1)
try:
    import click_log
except ImportError as e:
    print(
        "cli: click-log is not installed, please install click_log as it is one of the requirements."
    )
    exit(1)

"""
cli
~~~~~~~~~~~~~~~
:Description: console script for running genotyping
"""
"""
Created on January 29, 2020
Description: console script for running genotyping
@author: Ronak H Shah
"""

version = None
scriptpath = os.path.realpath(__file__)
p_scriptpath = pathlib.Path(scriptpath)
with open(os.path.join(p_scriptpath.parent, "__init__.py"), "r") as f:
    for line in f.readlines():
        line = line.strip()
        if line.startswith("__version__"):
            version = line.split("=")[-1].strip()
__all__ = []
__version__ = version
__date__ = "2020-01-29"
__updated__ = "2020-01-30"

plugin_folder = os.path.join(os.path.dirname(__file__), 'commands')

class MyCLI(click.MultiCommand):

    def list_commands(self, ctx):
        """Dynamically get the list of commands."""
        rv = []
        for filename in os.listdir(plugin_folder):
            if filename.endswith('.py') and not filename.startswith('__init__'):
                rv.append(filename[:-3])
        rv.sort()
        return rv

    def get_command(self, ctx, name):
        """Dynamically get the command."""
        ns = {}
        fn = os.path.join(plugin_folder, name + '.py')
        with open(fn) as f:
            code = compile(f.read(), fn, 'exec')
            eval(code, ns, ns)
        return ns['cli']


@click.command(cls=MyCLI)
@click.version_option(None, "-v", "--version", message="%(version)s", prog_name="genotype_variants")
def main(args=None):
    """Console script for genotype_variants."""
    pass


if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover



import logging
import numpy as np
import sys

"""
create_duplex_simplex_dataframe
~~~~~~~~~~~~~~~
:Description: Code to merge duplex and simplex fragment counts in MAF format
"""
"""
Created on February 10, 2020
Description: Code to merge duplex and simplex fragment counts in MAF format
@author: Ronak H Shah & Maysun Hasan
"""
# Making logging possible
logger = logging.getLogger("genotype_variants")
logger.info(
    "genotype:variants:small_variants::create_duplex_simplex_dataframe:: Generating duplex simplex dataframe"
)
# Adopted from Maysun script
def create_duplex_simplex_dataframe(simplex_dataframe, duplex_dataframe):
    """Code to merge duplex and simplex fragment counts in MAF format
    """
    np.seterr(divide="ignore", invalid="ignore")
    mutation_key = [
        "Chromosome",
        "Start_Position",
        "End_Position",
        "Reference_Allele",
        "Tumor_Seq_Allele2",
    ]
    df_s = simplex_dataframe.copy()
    df_d = duplex_dataframe.copy()

    # Prep Simplex
    try:
        df_s.rename(
            columns={
                "t_alt_count_fragment": "t_alt_count_fragment_simplex",
                "t_ref_count_fragment": "t_ref_count_fragment_simplex",
                "t_total_count_fragment": "t_total_count_fragment_simplex",
            },
            inplace=True,
        )
        logger.debug(
            "genotype:variants:small_variants:create_duplex_simplex_dataframe::Successfully renamed column names in simplex data frame"
        )
    except:
        e = sys.exc_info()[0]
        logger.error(
            "genotype:variants:small_variants::create_duplex_simplex_dataframe:: Could not rename column names in simplex data frame due to error, %s",
            e,
        )
        exit(1)

    try:
        df_s["Tumor_Seq_Allele2"] = df_s["Tumor_Seq_Allele1"]
        logger.debug(
            "genotype:variants:small_variants::create_duplex_simplex_dataframe:: Successfully generated Tumor_Seq_Allele2 column"
        )
    except:
        e = sys.exc_info()[0]
        logger.error(
            "genotype:variants:small_variants::create_duplex_simplex_dataframe:: Could not generate Tumor_Seq_Allele2 column due to error, %s",
            e,
        )
        exit(1)

    if df_s.shape[0] > 0:
        try:
            df_s["t_total_count_fragment_simplex"] = (
                df_s["t_ref_count_fragment_simplex"] + df_s["t_alt_count_fragment_simplex"]
            )
            logger.debug(
                "genotype:variants:small_variants::create_duplex_simplex_dataframe:: Successfully generated t_total_count_fragment_simplex column"
            )
        except:
            e = sys.exc_info()[0]
            logger.error(
                "genotype:variants:small_variants::create_duplex_simplex_dataframe:: Could not generate t_total_count_fragment_simplex column due to error, %s",
                e
            )
            exit(1)
    else:
        df_s["t_total_count_fragment_simplex"] = []

    if df_s.shape[0] > 0:
        try:
            df_s["t_vaf_fragment_simplex"] = (
                df_s["t_alt_count_fragment_simplex"]
                / (
                    df_s["t_alt_count_fragment_simplex"].astype(int)
                    + df_s["t_ref_count_fragment_simplex"].astype(int)
                )
            ).round(4)
            df_s["t_vaf_fragment_simplex"].fillna(0, inplace=True)
            logger.debug(
                "genotype:variants:small_variants:create_duplex_simplex_dataframe:: Successfully generated t_vaf_fragment_simplex column"
            )
        except:
            e = sys.exc_info()[0]
            logger.error(
                "genotype:variants:small_variants::create_duplex_simplex_dataframe:: Could not generate t_vaf_fragment_simplex column due to error, %s",
                e
            )
            exit(1)
    else:
        df_s["t_vaf_fragment_simplex"] = []

    try:
        df_s["Tumor_Sample_Barcode"] = df_s["Tumor_Sample_Barcode"].str.replace(
            "-SIMPLEX", ""
        )
        logger.debug(
            "genotype_variants:small_variants:create_duplex_simplex_dataframe:: Successfully renamed samples in Tumor_Sample_Barcode for duplex data frame"
        )
    except:
        e = sys.exc_info()[0]
        logger.error(
            "genotype:variants:small_variants:create_duplex_simplex_dataframe:: Could not rename samples in Tumor_Sample_Barcode for simplex data frame, due to error, %s",
            e,
        )

    try:
        df_s.set_index(mutation_key, drop=False, inplace=True)
        logger.debug(
            "genotype:variants:small_variants:create_duplex_simplex_dataframe:: Successfully reset the index for simplex data frame"
        )
    except:
        e = sys.exc_info()[0]
        logger.error(
            "genotype:variants:small_variants:create_duplex_simplex_dataframe:: Could not reset the index for simplex data frame, due to error, %s",
            e,
        )

    # Prep Duplex
    try:
        df_d.rename(
            columns={
                "t_alt_count_fragment": "t_alt_count_fragment_duplex",
                "t_ref_count_fragment": "t_ref_count_fragment_duplex",
                "t_total_count_fragment": "t_total_count_fragment_duplex",
            },
            inplace=True,
        )
        logger.debug(
            "genotype:variants:small_variants:create_duplex_simplex_dataframe::Successfully renamed column names in duplex data frame"
        )
    except:
        e = sys.exc_info()[0]
        logger.error(
            "genotype:variants:small_variants::create_duplex_simplex_dataframe:: Could not rename column names in duplex data frame due to error, %s",
            e,
        )
        exit(1)

    try:
        df_d["Tumor_Seq_Allele2"] = df_d["Tumor_Seq_Allele1"]
        logger.debug(
            "genotype:variants:small_variants::create_duplex_simplex_dataframe:: Successfully generated Tumor_Seq_Allele2 column"
        )
    except:
        e = sys.exc_info()[0]
        logger.error(
            "genotype:variants:small_variants::create_duplex_simplex_dataframe:: Could not generate Tumor_Seq_Allele2 column due to error, %s",
            e,
        )
        exit(1)

    if df_d.shape[0] > 0:
        try:
            df_d["t_total_count_fragment_duplex"] = (
                df_d["t_ref_count_fragment_duplex"] + df_d["t_alt_count_fragment_duplex"]
            )
            logger.debug(
                "genotype:variants:small_variants::create_duplex_simplex_dataframe:: Successfully generated t_total_count_fragment_duplex column"
            )
        except:
            e = sys.exc_info()[0]
            logger.error(
                "genotype:variants:small_variants::create_duplex_simplex_dataframe:: Could not generate t_total_count_fragment_duplex column due to error, %s",
                e
            )
            exit(1)
    else:
        df_d["t_total_count_fragment_duplex"] = []

    if df_d.shape[0] > 0:
        try:
            df_d["t_vaf_fragment_duplex"] = (
                df_d["t_alt_count_fragment_duplex"]
                / (
                    df_d["t_alt_count_fragment_duplex"].astype(int)
                    + df_d["t_ref_count_fragment_duplex"].astype(int)
                )
            ).round(4)
            df_d["t_vaf_fragment_duplex"].fillna(0, inplace=True)
            logger.debug(
                "genotype:variants:small_variants:create_duplex_simplex_dataframe:: Successfully generated t_vaf_fragment_duplex column"
            )
        except:
            e = sys.exc_info()[0]
            logger.error(
                "genotype:variants:small_variants::create_duplex_simplex_dataframe:: Could not generate t_vaf_fragment_duplex column due to error, %s",
                e
            )
            exit(1)
    else:
        df_d["t_vaf_fragment_duplex"] = []

    try:
        df_d["Tumor_Sample_Barcode"] = df_d["Tumor_Sample_Barcode"].str.replace(
            "-DUPLEX", ""
        )
        logger.debug(
            "genotype_variants:small_variants:create_duplex_simplex_dataframe:: Successfully renamed samples in Tumor_Sample_Barcode for duplex data frame"
        )
    except:
        e = sys.exc_info()[0]
        logger.error(
            "genotype:variants:small_variants:create_duplex_simplex_dataframe:: Could not rename samples in Tumor_Sample_Barcode for duplex data frame, due to error, %s",
            e,
        )

    try:
        df_d.set_index(mutation_key, drop=False, inplace=True)
        logger.debug(
            "genotype:variants:small_variants:create_duplex_simplex_dataframe:: Successfully reset the index for duplex data frame"
        )
    except:
        e = sys.exc_info()[0]
        logger.error(
            "genotype:variants:small_variants:create_duplex_simplex_dataframe:: Could not reset the index for duplex data frame, due to error, %s",
            e,
        )

    # Merge
    df_ds = None
    try:
        df_d = df_d.reindex(df_s.index)
        df_ds = df_s.merge(
            df_d[
                [
                    "t_ref_count_fragment_duplex",
                    "t_alt_count_fragment_duplex",
                    "t_total_count_fragment_duplex",
                    "t_vaf_fragment_duplex",
                ]
            ],
            left_index=True,
            right_index=True,
        )
        logger.info(
            "genotype_variants:small_variants:create_duplex_simplex_dataframe:: Successfully created merge data frame for simplex and duplex data"
        )
    except:
        e = sys.exc_info()[0]
        logger.error(
            "genotype:variants:small_variants:create_duplex_simplex_dataframe:: Could not create merge data frame for simplex and duplex data due to error, %s",
            e,
        )
        exit(1)

    ##Add
    if df_ds.shape[0] > 0:
        try:
            df_ds["t_ref_count_fragment_simplex_duplex"] = (
                df_ds["t_ref_count_fragment_simplex"] + df_ds["t_ref_count_fragment_duplex"]
            )
            df_ds["t_alt_count_fragment_simplex_duplex"] = (
                df_ds["t_alt_count_fragment_simplex"] + df_ds["t_alt_count_fragment_duplex"]
            )
            df_ds["t_total_count_fragment_simplex_duplex"] = (
                df_ds["t_alt_count_fragment_simplex_duplex"]
                + df_ds["t_ref_count_fragment_simplex_duplex"]
            )
            df_ds["t_vaf_fragment_simplex_duplex"] = (
                df_ds["t_alt_count_fragment_simplex_duplex"]
                / (
                    df_ds["t_alt_count_fragment_simplex_duplex"].astype(int)
                    + df_ds["t_ref_count_fragment_simplex_duplex"].astype(int)
                )
            ).round(4)
            df_ds["t_vaf_fragment_simplex_duplex"].fillna(0, inplace=True)
            logger.debug(
                "genotype_variants:small_variants:create_duplex_simplex_dataframe:: Successfully generated column for merged counts"
            )
        except:
            e = sys.exc_info()[0]
            logger.error(
                "genotype:variants:small_variants:create_duplex_simplex_dataframe:: Could not generate merged count column in the merged data frame due to error, %s",
                e
            )
            exit(1)
    else:
        df_ds["t_ref_count_fragment_simplex_duplex"] = []
        df_ds["t_alt_count_fragment_simplex_duplex"] = []
        df_ds["t_total_count_fragment_simplex_duplex"] = []
        df_ds["t_vaf_fragment_simplex_duplex"] = []

    ##clean up
    try:
        df_ds.drop(
            [
                "t_ref_count",
                "t_ref_count_forward",
                "t_alt_count",
                "t_alt_count_forward",
                "t_total_count",
                "t_total_count_forward",
                "t_variant_frequency",
            ],
            axis=1,
            inplace=True,
        )
        logger.debug(
            "genotype:variants:small_variants:create_duplex_simplex_dataframe:: Successfully dropped columns after merge"
        )
    except:
        e = sys.exc_info()[0]
        logger.error(
            "genotype:variants:small_variants:create_duplex_simplex_dataframe:: Could not drop columns after merge, %s",
            e,
        )
        exit(1)

    # Rename Sample Names
    try:
        df_ds["Tumor_Sample_Barcode"] = (
            df_ds["Tumor_Sample_Barcode"] + "-SIMPLEX-DUPLEX"
        )
        logger.debug(
            "genotype_variants:small_variants:create_duplex_simplex_dataframe:: Successfully renamed samples in Tumor_Sample_Barcode for merged data frame"
        )
    except:
        e = sys.exc_info()[0]
        logger.error(
            "genotype:variants:small_variants:create_duplex_simplex_dataframe:: Could not rename samples in Tumor_Sample_Barcode for merged data frame, due to error, %s",
            e,
        )
        exit(1)

    try:
        df_ds.set_index(mutation_key, drop=False, inplace=True)
        logger.debug(
            "genotype:variants:small_variants:create_duplex_simplex_dataframe:: Successfully reset the index for merged data frame"
        )
    except:
        e = sys.exc_info()[0]
        logger.error(
            "genotype:variants:small_variants:create_duplex_simplex_dataframe:: Could not reset the index for merged data frame, due to error, %s",
            e,
        )
        exit(1)

    logger.info(
        "Successfully merged data frame and the counts for simplex and duplex MAF"
    )
    return df_ds



import logging
import numpy as np
import sys

"""
create_all_maf_dataframe
~~~~~~~~~~~~~~~
:Description: Code to merge all the data frames generated from MAF files into one data frame
"""
"""
Created on February 10, 2020
Description: Code to merge all the data frames generated from MAF files into one data frame
@author: Ronak H Shah & Maysun Hasan
"""
# Making logging possible
logger = logging.getLogger("genotype_variants")
logger.info(
    "genotype:variants:small_variants::create_all_maf_dataframe:: Generating duplex simplex dataframe"
)
# Adopted from Maysun script
def create_all_maf_dataframe(
    original_dataframe, standard_dataframe, simplex_duplex_dataframe
):
    """Code to merge all the data frames generated from MAF files into one data frame
    """
    np.seterr(divide="ignore", invalid="ignore")
    mutation_key = [
        "Chromosome",
        "Start_Position",
        "End_Position",
        "Reference_Allele",
        "Tumor_Seq_Allele2",
    ]
    (df_ds, df_s, df_o) = None, None, None
    if simplex_duplex_dataframe is not None:
        df_ds = simplex_duplex_dataframe.copy()
    if standard_dataframe is not None:
        df_s = standard_dataframe.copy()
    if original_dataframe is not None:
        df_o = original_dataframe.copy()

    # Prep Simplex duplex
    if df_ds is not None:
        try:
            df_ds["Tumor_Sample_Barcode"] = df_ds["Tumor_Sample_Barcode"].str.replace(
                "-SIMPLEX-DUPLEX", ""
            )
            logger.debug(
                "genotype_variants:small_variants:create_all_maf_dataframe:: Successfully renamed samples in Tumor_Sample_Barcode for simplex duplex data frame"
            )
        except:
            e = sys.exc_info()[0]
            logger.error(
                "genotype:variants:small_variants:create_all_maf_dataframe:: Could not rename samples in Tumor_Sample_Barcode for simplex duplex data frame, due to error, %s",
                e,
            )
            exit(1)

        try:
            df_ds.set_index(mutation_key, drop=False, inplace=True)
            logger.debug(
                "genotype:variants:small_variants:create_all_maf_dataframe:: Successfully reset the index for simplex duplex data frame"
            )
        except:
            e = sys.exc_info()[0]
            logger.error(
                "genotype:variants:small_variants:create_all_maf_dataframe:: Could not reset the index for simplex duplex data frame, due to error, %s",
                e,
            )
            exit(1)

    # Prep Standard
    if df_s is not None:
        try:
            df_s.rename(
                columns={
                    "t_alt_count": "t_alt_count_standard",
                    "t_ref_count": "t_ref_count_standard",
                    "t_total_count": "t_total_count_standard",
                    "t_variant_frequency": "t_variant_frequency_standard",
                    "t_alt_count_forward": "t_alt_count_forward_standard",
                    "t_ref_count_forward": "t_ref_count_forward_standard",
                    "t_total_count_forward": "t_total_count_forward_standard",
                },
                inplace=True,
            )
            logger.debug(
                "genotype:variants:small_variants:create_all_maf_dataframe::Successfully renamed column names in standard data frame"
            )
        except:
            e = sys.exc_info()[0]
            logger.error(
                "genotype:variants:small_variants::create_all_maf_dataframe:: Could not rename column names in standard data frame due to error, %s",
                e,
            )
            exit(1)

        try:
            df_s["Tumor_Seq_Allele2"] = df_s["Tumor_Seq_Allele1"]
            logger.debug(
                "genotype:variants:small_variants::create_duplex_simplex_dataframe:: Successfully generated Tumor_Seq_Allele2 column"
            )
        except:
            e = sys.exc_info()[0]
            logger.error(
                "genotype:variants:small_variants::create_duplex_simplex_dataframe:: Could not generate Tumor_Seq_Allele2 column due to error, %s",
                e,
            )
            exit(1)

        if df_s.shape[0] > 0:
            try:
                df_s["t_alt_count_reverse_standard"] = (
                    df_s["t_alt_count_standard"]
                    - df_s["t_alt_count_forward_standard"]
                )
                df_s["t_ref_count_reverse_standard"] = (
                    df_s["t_ref_count_standard"]
                    - df_s["t_ref_count_forward_standard"]
                )
                df_s["t_total_count_reverse_standard"] = (
                    df_s["t_total_count_standard"] - df_s["t_total_count_forward_standard"]
                )
                logger.debug(
                    "genotype:variants:small_variants::create_all_maf_dataframe:: Successfully generated reverse count columns in standard data frame"
                )
            except:
                e = sys.exc_info()[0]
                logger.error(
                    "genotype:variants:small_variants::create_all_maf_dataframe:: Could not generate reverse count columns in standard data frame due to error, %s",
                    e
                )
                exit(1)
        else:
            df_s["t_alt_count_reverse_standard"] = []
            df_s["t_ref_count_reverse_standard"] = []
            df_s["t_total_count_reverse_standard"] = []

        try:
            df_s["Tumor_Sample_Barcode"] = df_s["Tumor_Sample_Barcode"].str.replace(
                "-STANDARD", ""
            )
            logger.debug(
                "genotype_variants:small_variants:create_all_maf_dataframe:: Successfully renamed samples in Tumor_Sample_Barcode for standard data frame"
            )
        except:
            e = sys.exc_info()[0]
            logger.error(
                "genotype:variants:small_variants:create_all_maf_dataframe:: Could not rename samples in Tumor_Sample_Barcode for standard data frame, due to error, %s",
                e,
            )
            exit(1)

        try:
            df_s.set_index(mutation_key, drop=False, inplace=True)
            logger.debug(
                "genotype:variants:small_variants:create_all_maf_dataframe:: Successfully reset the index for standard data frame"
            )
        except:
            e = sys.exc_info()[0]
            logger.error(
                "genotype:variants:small_variants:create_all_maf_dataframe:: Could not reset the index for standard data frame, due to error, %s",
                e,
            )
            exit(1)

    # Prep Original
    if df_o is not None:
        try:
            df_o.set_index(mutation_key, drop=False, inplace=True)
            logger.debug(
                "genotype:variants:small_variants:create_all_maf_dataframe:: Successfully reset the index for original data frame"
            )
        except:
            e = sys.exc_info()[0]
            logger.error(
                "genotype:variants:small_variants:create_all_maf_dataframe:: Could not reset the index for original data frame, due to error, %s",
                e,
            )
            exit(1)

    # Merge STANDARD with DUPLEX-SIMPLEX data frame
    df_s_ds = None
    if df_ds is not None and df_s is not None:
        try:
            df_ds = df_ds.reindex(df_s.index)
            df_s_ds = df_s.merge(
                df_ds[
                    [
                        "t_ref_count_fragment_simplex",
                        "t_alt_count_fragment_simplex",
                        "t_total_count_fragment_simplex",
                        "t_vaf_fragment_simplex",
                        "t_ref_count_fragment_duplex",
                        "t_alt_count_fragment_duplex",
                        "t_total_count_fragment_duplex",
                        "t_vaf_fragment_duplex",
                        "t_ref_count_fragment_simplex_duplex",
                        "t_alt_count_fragment_simplex_duplex",
                        "t_total_count_fragment_simplex_duplex",
                        "t_vaf_fragment_simplex_duplex",
                    ]
                ],
                left_index=True,
                right_index=True,
            )
            logger.info(
                "genotype_variants:small_variants:create_all_maf_dataframe:: Successfully created merge data frame for standard, simplex and duplex data"
            )
        except:
            e = sys.exc_info()[0]
            logger.error(
                "genotype:variants:small_variants:create_all_maf_dataframe:: Could not create merge data frame for standard, simplex and duplex data due to error, %s",
                e,
            )
            exit(1)
        try:
            df_s_ds.set_index(mutation_key, drop=False, inplace=True)
            logger.debug(
                "genotype:variants:small_variants:create_all_maf_dataframe:: Successfully reset the index for merged data frame"
            )
        except:
            e = sys.exc_info()[0]
            logger.error(
                "genotype:variants:small_variants:create_all_maf_dataframe:: Could not reset the index for merged data frame, due to error, %s",
                e,
            )
            exit(1)

    # Merge Original with STANDARD-DUPLEX-SIMPLEX data frame
    df_o_s_ds = None
    if df_s_ds is not None:
        try:
            df_s_ds = df_s_ds.reindex(df_o.index)
            df_o_s_ds = df_o.merge(
                df_s_ds[
                    [
                        "t_ref_count_standard",
                        "t_alt_count_standard",
                        "t_total_count_standard",
                        "t_variant_frequency_standard",
                        "t_ref_count_forward_standard",
                        "t_alt_count_forward_standard",
                        "t_total_count_forward_standard",
                        "t_ref_count_reverse_standard",
                        "t_alt_count_reverse_standard",
                        "t_total_count_reverse_standard",
                        "t_ref_count_fragment_simplex",
                        "t_alt_count_fragment_simplex",
                        "t_total_count_fragment_simplex",
                        "t_vaf_fragment_simplex",
                        "t_ref_count_fragment_duplex",
                        "t_alt_count_fragment_duplex",
                        "t_total_count_fragment_duplex",
                        "t_vaf_fragment_duplex",
                        "t_ref_count_fragment_simplex_duplex",
                        "t_alt_count_fragment_simplex_duplex",
                        "t_total_count_fragment_simplex_duplex",
                        "t_vaf_fragment_simplex_duplex",
                    ]
                ],
                left_index=True,
                right_index=True,
            )
            logger.info(
                "genotype_variants:small_variants:create_all_maf_dataframe:: Successfully created merge data frame for original, standard, simplex and duplex data"
            )
        except:
            e = sys.exc_info()[0]
            logger.error(
                "genotype:variants:small_variants:create_all_maf_dataframe:: Could not create merge data frame for original, standard, simplex and duplex data due to error, %s",
                e,
            )
            exit(1)

        try:
            df_o_s_ds.set_index(mutation_key, drop=False, inplace=True)
            logger.debug(
                "genotype:variants:small_variants:create_all_maf_dataframe:: Successfully reset the index for merged data frame"
            )
        except:
            e = sys.exc_info()[0]
            logger.error(
                "genotype:variants:small_variants:create_all_maf_dataframe:: Could not reset the index for merged data frame, due to error, %s",
                e,
            )
            exit(1)

    # Merge Original MAF data frame with STANDARD MAF data frame
    df_o_s = None
    if df_o is not None and df_s is not None:
        try:
            df_s = df_s.reindex(df_o.index)
            df_o_s = df_o.merge(
                df_s[
                    [
                        "t_ref_count_standard",
                        "t_alt_count_standard",
                        "t_total_count_standard",
                        "t_variant_frequency_standard",
                        "t_ref_count_forward_standard",
                        "t_alt_count_forward_standard",
                        "t_total_count_forward_standard",
                        "t_ref_count_reverse_standard",
                        "t_alt_count_reverse_standard",
                        "t_total_count_reverse_standard",
                    ]
                ],
                left_index=True,
                right_index=True,
            )
            logger.info(
                "genotype_variants:small_variants:create_all_maf_dataframe:: Successfully created merge data frame for original, standard, simplex and duplex data"
            )
        except:
            e = sys.exc_info()[0]
            logger.error(
                "genotype:variants:small_variants:create_all_maf_dataframe:: Could not create merge data frame for original, standard, simplex and duplex data due to error, %s",
                e,
            )
            exit(1)

        try:
            df_o_s.set_index(mutation_key, drop=False, inplace=True)
            logger.debug(
                "genotype:variants:small_variants:create_all_maf_dataframe:: Successfully reset the index for merged data frame"
            )
        except:
            e = sys.exc_info()[0]
            logger.error(
                "genotype:variants:small_variants:create_all_maf_dataframe:: Could not reset the index for merged data frame, due to error, %s",
                e,
            )
            exit(1)

    # Merge original maf data frame with duplex and simplex and duplex data frame
    df_o_ds = None
    if df_ds is not None and df_o is not None:
        try:
            df_ds = df_ds.reindex(df_o.index)
            df_o_ds = df_o.merge(
                df_ds[
                    [
                        "t_ref_count_fragment_simplex",
                        "t_alt_count_fragment_simplex",
                        "t_total_count_fragment_simplex",
                        "t_vaf_fragment_simplex",
                        "t_ref_count_fragment_duplex",
                        "t_alt_count_fragment_duplex",
                        "t_total_count_fragment_duplex",
                        "t_vaf_fragment_duplex",
                        "t_ref_count_fragment_simplex_duplex",
                        "t_alt_count_fragment_simplex_duplex",
                        "t_total_count_fragment_simplex_duplex",
                        "t_vaf_fragment_simplex_duplex",
                    ]
                ],
                left_index=True,
                right_index=True,
            )
            logger.info(
                "genotype_variants:small_variants:create_all_maf_dataframe:: Successfully created merge data frame for original, simplex and duplex data"
            )
        except:
            e = sys.exc_info()[0]
            logger.error(
                "genotype:variants:small_variants:create_all_maf_dataframe:: Could not create merge data frame for original, simplex and duplex data due to error, %s",
                e,
            )
            exit(1)

        try:
            df_o_ds.set_index(mutation_key, drop=False, inplace=True)
            logger.debug(
                "genotype:variants:small_variants:create_all_maf_dataframe:: Successfully reset the index for merged data frame"
            )
        except:
            e = sys.exc_info()[0]
            logger.error(
                "genotype:variants:small_variants:create_all_maf_dataframe:: Could not reset the index for merged data frame, due to error, %s",
                e,
            )
            exit(1)

    logger.info("Successfully merged data frame")

    if df_o_s_ds is not None:
        return df_o_s_ds
    elif df_o_ds is not None:
        return df_o_ds
    elif df_o_s is not None:
        return df_o_s
    else:
        return df_s_ds
